/* 
   SquidTL - Time Limit for Squid proxy
   Copyright (C) 2009-2010, Michele Pinassi <o-zone@zerozone.it>.
   
   http://www.zerozone.it/Software/Linux/SquidTL
                                                                     
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
  
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "common.h"

static void print_element_names(xmlNode *a_node) {
    xmlNode *cur_node = NULL;

    for(cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if(cur_node->type == XML_ELEMENT_NODE) {	    
            LOG_Write(DEBUG,"Node %s -> %s\n", cur_node->name,xmlNodeGetContent(cur_node));
        }
        print_element_names(cur_node->children);
    }
}

struct _Config *MyConfig=NULL;

char *atohx(char *dst, const char *src)
{       
    char *ret = dst;
    int lsb, msb;
    
    for(lsb, msb; *src; src += 2) {
        msb = tolower(*src);
        lsb = tolower(*(src + 1));
        msb -= isdigit(msb) ? 0x30 : 0x57;
        lsb -= isdigit(lsb) ? 0x30 : 0x57;
        if((msb < 0x0 || msb > 0xf) || (lsb < 0x0 || lsb > 0xf)) {
            *ret = 0;
            return NULL;
        }
        *dst++ = (char)(lsb | (msb << 4));  
    }
    *dst = 0;
    return ret;
}

void CONFIG_ParseCommonSection(struct _Config *MyConfig,xmlNode *a_node) {
    xmlNode *cur_node = NULL;
    for(cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if(cur_node->type == XML_ELEMENT_NODE) {	    
            LOG_Write(DEBUG,"COMMON Node %s -> %s\n", cur_node->name,xmlNodeGetContent(cur_node));
    	    if(strcasecmp(cur_node->name,"defaultmode") == 0) {
    		strncpy(MyConfig->defaultMode,xmlNodeGetContent(cur_node),32);
	    }
    	    if(strcasecmp(cur_node->name,"defaultaction") == 0) {
    		strncpy(MyConfig->defaultAction,xmlNodeGetContent(cur_node),32);
	    }
    	    if(strcasecmp(cur_node->name,"defaultredirecturl") == 0) {
    		strncpy(MyConfig->defaultRedirectURL,xmlNodeGetContent(cur_node),255);
	    }
    	    if(strcasecmp(cur_node->name,"defaultuseraction") == 0) {
    		strncpy(MyConfig->defaultUserAction,xmlNodeGetContent(cur_node),32);
	    }
    	    if(strcasecmp(cur_node->name,"defaultuserlevel") == 0) {
    		MyConfig->defaultUserLevel = atoi(xmlNodeGetContent(cur_node));
	    }
    	    if(strcasecmp(cur_node->name,"defaultusergroup") == 0) {
    		MyConfig->defaultUserGroup = atoi(xmlNodeGetContent(cur_node));
	    }
    	    if(strcasecmp(cur_node->name,"urllistpath") == 0) {
    		strncpy(MyConfig->URLListPath,xmlNodeGetContent(cur_node),255);
	    }
    	    if(strcasecmp(cur_node->name,"xmlrpcserver") == 0) {
    		strncpy(MyConfig->xmlrpcServer,xmlNodeGetContent(cur_node),255);
	    }
    	    if(strcasecmp(cur_node->name,"purgetimeout") == 0) {
    		MyConfig->purgeTimeout = atoi(xmlNodeGetContent(cur_node));
	    }    
	}
    }
}

void CONFIG_ParseMySQLSection(struct _Config *MyConfig,xmlNode *a_node) {
    xmlNode *cur_node = NULL;
    for(cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if(cur_node->type == XML_ELEMENT_NODE) {	    
            LOG_Write(DEBUG,"MySQL Node %s -> %s\n", cur_node->name,xmlNodeGetContent(cur_node));
    	    if(strcasecmp(cur_node->name,"host") == 0) {
    		strncpy(MyConfig->mysqlHost,xmlNodeGetContent(cur_node),32);
	    }
    	    if(strcasecmp(cur_node->name,"port") == 0) {
    		strncpy(MyConfig->mysqlPort,xmlNodeGetContent(cur_node),16);
	    }
    	    if(strcasecmp(cur_node->name,"db") == 0) {
    		strncpy(MyConfig->mysqlDb,xmlNodeGetContent(cur_node),32);
	    }
    	    if(strcasecmp(cur_node->name,"user") == 0) {
    		strncpy(MyConfig->mysqlUser,xmlNodeGetContent(cur_node),32);
	    }
    	    if(strcasecmp(cur_node->name,"password") == 0) {
    		strncpy(MyConfig->mysqlPassword,xmlNodeGetContent(cur_node),32);
	    }
	}
    }
}

void CONFIG_ParseConfig(struct _Config *MyConfig,xmlNode *a_node) {
    struct _X10Item *X10Item=NULL;
    struct _Zone *TempZone=NULL;
    struct _Trigger *TempTrigger=NULL;
    struct _Action *TempAction=NULL;
    xmlNode *cur_node = NULL;
        
    for(cur_node = a_node; cur_node; cur_node = cur_node->next) {
        if(cur_node->type == XML_ELEMENT_NODE) {	    
    	    if(strcasecmp(cur_node->name,"common") == 0) {
    		CONFIG_ParseCommonSection(MyConfig, cur_node->children);
    	    }
    	    if(strcasecmp(cur_node->name,"mysql") == 0) {
    		CONFIG_ParseMySQLSection(MyConfig, cur_node->children);
    	    }
    	}
    }
}

/* Load config.xml */
struct _Config *CONFIG_Load(char *configFile) {
    xmlDoc *doc = NULL;
    xmlNode *root_element = NULL;
    xmlNode *cur_node = NULL;

    LOG_Write(DEBUG,"Load configuration from %s\n",configFile);

    /* Ok, now parse XML... */
    LIBXML_TEST_VERSION

    doc = xmlReadFile(configFile, NULL, 0);

    if (doc == NULL) {
        LOG_Write(ERROR,"Could not parse config file %s\n", configFile);
        return -1;
    }

    /* Allocate memory for config and zero it ! */
    MyConfig = malloc(sizeof(struct _Config));
    bzero(MyConfig,sizeof(struct _Config));

    /* Set default values */
    strncpy(MyConfig->defaultMode,"CHECK",32);
    strncpy(MyConfig->defaultAction,"ALLOW",32);
    strncpy(MyConfig->defaultUserAction,"PASS",32);
    MyConfig->purgeTimeout = 10080; /* 7 days */

    /*Get the root element node */
    root_element = xmlDocGetRootElement(doc);

    for(cur_node = root_element; cur_node; cur_node = cur_node->next) {
        if(cur_node->type == XML_ELEMENT_NODE) {	    
    	    if(strcasecmp(cur_node->name,"config") == 0) {
    		CONFIG_ParseConfig(MyConfig,cur_node->children);
    	    }
        }
    }
    xmlFreeDoc(doc);
    xmlCleanupParser();
    return MyConfig;
}

